@@ -1,7 +1,7 @@
 /*
  * Part of DNS zone file validator `validns`.
  *
- * Copyright 2025 OARC, Inc.
+ * Copyright 2025-2026 OARC, Inc.
  * Copyright 2011-2025 Anton Berezin <tobez@tobez.org>
  * Modified BSD license.
  * (See LICENSE file in the distribution.)
@@ -12,8 +12,8 @@
 
 const char *thisprogname(void);
 
-void croak(int exit_code, const char *fmt, ...);
-void croakx(int exit_code, const char *fmt, ...);
+void croak(int exit_code, const char *fmt, ...) __attribute__((noreturn));
+void croakx(int exit_code, const char *fmt, ...) __attribute__((noreturn));
 
 void *bitch(const char *fmt, ...);
 void *moan(char *file_name, int line, const char *fmt, ...);
