@@ -1,7 +1,7 @@
 /*
  * Part of DNS zone file validator `validns`.
  *
- * Copyright 2025 OARC, Inc.
+ * Copyright 2025-2026 OARC, Inc.
  * Copyright 2011-2025 Anton Berezin <tobez@tobez.org>
  * Modified BSD license.
  * (See LICENSE file in the distribution.)
@@ -213,7 +213,7 @@ int name_belongs_to_zone(const char *name)
     if (zone_apex && name_l >= zone_apex_l) {
         if (strcmp(zone_apex, name+name_l-zone_apex_l) != 0) {
             return 0;
-        } else if (name_l > zone_apex_l && name[name_l-zone_apex_l-1] != '.') {
+        } else if (name_l > zone_apex_l && zone_apex_l > 1 && name[name_l-zone_apex_l-1] != '.') {
             return 0;
         }
     } else {
@@ -268,7 +268,7 @@ struct rr *store_record(int rdtype, char *name, long ttl, void *rrptr)
     if (zone_apex && name_l >= zone_apex_l) {
         if (strcmp(zone_apex, name+name_l-zone_apex_l) != 0) {
             return bitch("%s does not belong to zone %s", name, zone_apex);
-        } else if (name_l > zone_apex_l && name[name_l-zone_apex_l-1] != '.') {
+        } else if (name_l > zone_apex_l && zone_apex_l > 1 && name[name_l-zone_apex_l-1] != '.') {
             return bitch("%s does not belong to zone %s", name, zone_apex);
         }
     } else {
@@ -823,7 +823,7 @@ void validate_zone(void)
     cbtree_allprefixed(&zone_data, "", validate_named_rr, NULL);
     cbtree_allprefixed(&zone_data, "", second_pass_one_name, NULL);
 
-    if (G.dnssec_active && !G.nsec3_present)
+    if (zone_apex && G.dnssec_active && !G.nsec3_present)
         validate_nsec_chain();
 }
 
