@@ -28,6 +28,20 @@ is(rc, 0, 'valid signed zone parses ok');
 run('./validns', @threads, $srcdir . '/t/zones/wallet.zone');
 is(rc, 0, 'WALLET records parse correctly');
 
+# Root zone test - TLDs should belong to the root zone
+run('./validns', @threads, '-z.', $srcdir . '/t/zones/root-zone');
+is(rc, 0, 'root zone with TLDs parses correctly');
+
+# Issue #86: KSK check with no DNSKEY records should not crash
+run('./validns', @threads, '-p', 'ksk-exists', $srcdir . '/t/zones/dnssec-no-dnskey.zone');
+isnt(rc, 11, 'ksk-exists check with no DNSKEY does not crash');
+like(stderr, qr/No DNSKEY found/, 'ksk-exists check reports missing DNSKEY');
+
+# Issue #86: NSEC3PARAM not-apex check with invalid record should not crash
+run('./validns', @threads, '-p', 'nsec3param-not-apex', $srcdir . '/t/zones/nsec3param-not-in-zone.zone');
+isnt(rc, 11, 'nsec3param-not-apex check with bad record does not crash');
+like(stderr, qr/does not belong to zone/, 'nsec3param-not-apex reports zone membership error');
+
 # SPF many segments test (>20)
 run('./validns', @threads, $srcdir . '/t/zones/spf-many-segments.zone');
 is(rc, 0, 'SPF records handle >20 segments');
@@ -435,6 +449,31 @@ like(shift @e, qr/wrong digest length, expected 48 bytes for hash algorithm 1/,
 like(shift @e, qr/unsupported hash algorithm 3 \(only 1=SHA-384 and 2=SHA-512 are defined\)/, "ZONEMD invalid hash algorithm");
 is(+@e, 0, "no unaccounted errors for ZONEMD checks");
 
+# Coredump test: malformed zone with DNSSEC records but no valid SOA should not crash
+run('./validns', @threads, $srcdir . '/t/issues/coredump-null-zone-apex/crash.zone');
+isnt(rc, 0, 'malformed zone with RRSIG returns an error');
+ok(rc == 0 || rc > 127, 'malformed zone with RRSIG does not crash (no signal)');
+@e = split /\n/, stderr;
+like(shift @e, qr/ttl not specified/, "RRSIG: ttl not specified");
+like(shift @e, qr/the first record in the zone must be an SOA record/, "RRSIG: first record not SOA");
+is(+@e, 0, "no unaccounted errors for RRSIG coredump check");
+
+run('./validns', @threads, $srcdir . '/t/issues/coredump-null-zone-apex/crash-nsec.zone');
+isnt(rc, 0, 'malformed zone with NSEC returns an error');
+ok(rc == 0 || rc > 127, 'malformed zone with NSEC does not crash (no signal)');
+@e = split /\n/, stderr;
+like(shift @e, qr/ttl not specified/, "NSEC: ttl not specified");
+like(shift @e, qr/the first record in the zone must be an SOA record/, "NSEC: first record not SOA");
+is(+@e, 0, "no unaccounted errors for NSEC coredump check");
+
+run('./validns', @threads, $srcdir . '/t/issues/coredump-null-zone-apex/crash-nsec3.zone');
+isnt(rc, 0, 'malformed zone with NSEC3 returns an error');
+ok(rc == 0 || rc > 127, 'malformed zone with NSEC3 does not crash (no signal)');
+@e = split /\n/, stderr;
+like(shift @e, qr/ttl not specified/, "NSEC3: ttl not specified");
+like(shift @e, qr/NSEC3 record name is not valid/, "NSEC3: record name not valid");
+is(+@e, 0, "no unaccounted errors for NSEC3 coredump check");
+
 }
 
 done_testing;
